==> easy_http-0.0.10.tar.gz: lib/http/easy_http_controller.dart <==
import 'dart:developer';

import 'package:easy_http/easy_http.dart';
import 'package:easy_http/http/easy_http_connect.dart';

abstract class EasyHttpController<T> extends GetxController with StateMixin<T> {
  T get initHttpResponseData;

  late EasyHttpConnect<T> _httpClient;

  EasyHttpConnect<T> get httpClient => _httpClient;

  late final Rx<T> _httpData = _httpClient.obsHttpData;

  T get httpData => _httpData.value;

  Rx<T> get obsHttpData => _httpData;

  String get localCacheKey => "";

  String get requestUrl;

  bool get needAuthorized => true;

  @override
  void onInit() {
    _httpClient = Get.put(
      EasyHttpConnect<T>(initHttpResponseData, localCacheKey: localCacheKey),
    );

    super.onInit();
  }

  Future<T> get({
    Map<String, String>? headers,
    String? contentType,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    if (EasyHttp.config.showLog) {
      log("request query = ${query.toString()}");
    }
    return onLoading(() async {
      try {
        final response = await _httpClient.get(
          requestUrl,
          headers: headers,
          contentType: contentType,
          query: query,
        );
        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> post(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    if (EasyHttp.config.showLog) {
      log("request body = ${body.toString()}");
    }
    return onLoading(() async {
      try {
        final response = await _httpClient.post(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> put(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.put(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> patch(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.patch(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> delete({
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.delete(
          requestUrl,
          headers: headers,
          query: query,
          contentType: contentType,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> onLoading(Future<T> Function() asyncFunction, {bool showDefaultLoading = true}) async {
    if (showDefaultLoading) {
      return Get.showOverlay(
        asyncFunction: asyncFunction,
        opacity: .1,
        loadingWidget: EasyHttp.config.loadingWidget,
      );
    } else {
      return asyncFunction();
    }
  }

  void onEmpty() {}

  void onSuccess() {}

  void onError([String? message]) {
    if (EasyHttp.config.showLog) {
      log(message ?? "error");
    }
  }
}

==> easy_http-0.0.11.tar.gz: lib/http/easy_http_controller.dart <==
import 'dart:developer';

import 'package:easy_http/easy_http.dart';
import 'package:easy_http/http/easy_http_connect.dart';

abstract class EasyHttpController<T> extends GetxController with StateMixin<T> {
  T get initHttpResponseData;

  late EasyHttpConnect<T> _httpClient;

  EasyHttpConnect<T> get httpClient => _httpClient;

  late final Rx<T> _httpData = _httpClient.obsHttpData;

  T get httpData => _httpData.value;

  Rx<T> get obsHttpData => _httpData;

  String get localCacheKey => "";

  String get requestUrl;

  bool get needAuthorized => true;

  @override
  void onInit() {
    //如此初始化会导致相同controller 不同tag的时候， localCacheKey 有问题，换成如下普通的方式
    // _httpClient = Get.put(
    //   EasyHttpConnect<T>(initHttpResponseData, localCacheKey: localCacheKey),
    // );
    _httpClient = EasyHttpConnect<T>(initHttpResponseData, localCacheKey: localCacheKey);
    _httpClient.onInit();

    super.onInit();
  }

  Future<T> get({
    Map<String, String>? headers,
    String? contentType,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    if (EasyHttp.config.showLog) {
      log("request query = ${query.toString()}");
    }
    return onLoading(() async {
      try {
        final response = await _httpClient.get(
          requestUrl,
          headers: headers,
          contentType: contentType,
          query: query,
        );
        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> post(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    if (EasyHttp.config.showLog) {
      log("request body = ${body.toString()}");
    }
    return onLoading(() async {
      try {
        final response = await _httpClient.post(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> put(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.put(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> patch(
    dynamic body, {
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.patch(
          requestUrl,
          body,
          contentType: contentType,
          headers: headers,
          query: query,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> delete({
    String? contentType,
    Map<String, String>? headers,
    Map<String, dynamic>? query,
    bool showDefaultLoading = true,
  }) async {
    return onLoading(() async {
      try {
        final response = await _httpClient.delete(
          requestUrl,
          headers: headers,
          query: query,
          contentType: contentType,
        );

        if (!response.isOk) {
          onError("${response.request?.url} ${response.statusText} \n${response.body}");
        } else {
          onSuccess();
        }
      } catch (e, _) {
        onError(e.toString());
        rethrow;
      }
      return httpData;
    }, showDefaultLoading: showDefaultLoading);
  }

  Future<T> onLoading(Future<T> Function() asyncFunction, {bool showDefaultLoading = true}) async {
    if (showDefaultLoading) {
      return Get.showOverlay(
        asyncFunction: asyncFunction,
        opacity: .1,
        loadingWidget: EasyHttp.config.loadingWidget,
      );
    } else {
      return asyncFunction();
    }
  }

  void onEmpty() {}

  void onSuccess() {}

  void onError([String? message]) {
    if (EasyHttp.config.showLog) {
      log(message ?? "error");
    }
  }
}
